# type: ignore
from __future__ import annotations

from pathlib import Path

from environs import Env

env = Env()
env.read_env()

PARE_API_URL: str = env.str("PARE_API_URL", "https://api.pare.gauge.sh")
PARE_API_VERSION: str = env.str("PARE_API_VERSION", "v0.1")
PARE_API_DEPLOY_URL_PATH: str = env.str("PARE_API_DEPLOY_URL_PATH", "/deploy/")
PARE_API_SERVICES_URL_PATH: str = env.str("PARE_API_SERVICES_URL_PATH", "/services/")
PARE_API_INVOKE_URL_PATH: str = env.str("PARE_API_INVOKE_URL_PATH", "/services/invoke/")
PARE_API_DELETE_URL_PATH: str = env.str("PARE_API_DELETE_URL_PATH", "/services/delete/")

PARE_API_KEY: str = env.str("PARE_API_KEY", "")
PARE_API_KEY_FILE: str = env.str("PARE_API_KEY_FILE", ".pare/api_key.priv")

if not PARE_API_KEY and PARE_API_KEY_FILE:
    pare_api_key_path = Path(PARE_API_KEY_FILE)
    if pare_api_key_path.exists():
        PARE_API_KEY = pare_api_key_path.read_text()

PARE_API_KEY_HEADER: str = env.str("PARE_API_KEY_HEADER", "X-Pare-API-Key")


PARE_ATOMIC_DEPLOYMENT_ENABLED: bool = env.bool("PARE_ATOMIC_DEPLOYMENT_ENABLED", False)
PARE_ATOMIC_DEPLOYMENT_HEADER: str = env.str(
    "PARE_ATOMIC_DEPLOYMENT_HEADER", "X-Pare-Atomic-Deployment"
)
PARE_GIT_HASH: str = env.str("PARE_GIT_HASH", "")

_KNOWN_GIT_HASH_ENV_VARS = [
    "GITHUB_SHA",  # GitHub Actions
    "CI_COMMIT_SHA",  # GitLab CI
    "GIT_COMMIT",  # Jenkins
    "CIRCLE_SHA1",  # CircleCI
    "TRAVIS_COMMIT",  # Travis CI
    "BUILD_SOURCEVERSION",  # Azure Pipelines
    "BITBUCKET_COMMIT",  # Bitbucket Pipelines
    "BUILD_VCS_NUMBER",  # TeamCity
    "DRONE_COMMIT_SHA",  # Drone CI
    "CODEBUILD_RESOLVED_SOURCE_VERSION",  # AWS CodeBuild
]

if not PARE_GIT_HASH:
    # try to get the git hash from the environment
    for env_var in _KNOWN_GIT_HASH_ENV_VARS:
        PARE_GIT_HASH = env.str(env_var, "")
        if PARE_GIT_HASH:
            break



from __future__ import annotations

import glob
import importlib.util
import inspect
import os
import sys
import tempfile
import zipfile
from pathlib import Path
from typing import cast

import requests
from rich.console import Console

from pare import settings
from pare.client import get_current_git_hash
from pare.console import log_error, log_task
from pare.constants import PYTHON_VERSION
from pare.models import DeployConfig, ServiceConfig, ServiceRegistration


class DeployHandler:
    def __init__(
        self,
        file_patterns: list[str],
        environment_variables: dict[str, str] | None = None,
    ) -> None:
        self.file_paths: set[Path] = set()
        for pattern in file_patterns:
            path = Path(pattern)
            if path.is_dir():
                self.file_paths.update(path.rglob("**/*.py"))
            else:
                self.file_paths.update(
                    Path(path)
                    for path in glob.glob(pattern, recursive=True)
                    if path.endswith(".py")
                )
        self.environment_variables = environment_variables or {}
        self.deploy_url = f"{settings.PARE_API_URL}/{settings.PARE_API_VERSION}{settings.PARE_API_DEPLOY_URL_PATH}"

    @property
    def headers(self) -> dict[str, str]:
        return {
            settings.PARE_API_KEY_HEADER: settings.PARE_API_KEY,
        }

    def validate_file_paths(self) -> None:
        errored = False
        for file_path in self.file_paths:
            if not file_path.exists():
                errored = True
                log_error(f"{file_path} does not exist")
            elif not file_path.is_file():
                errored = True
                log_error(f"{file_path} is not a file")
            elif file_path.suffix.lower() != ".py":
                errored = True
                log_error(f"{file_path} is not a python file")
        if errored:
            sys.exit(1)

    def bundle(self, temp_dir: str) -> Path:
        with log_task(start_message="Bundling...", end_message="Project bundled"):
            zip_filename = "pare_project_bundle.zip"
            zip_path = Path(temp_dir) / zip_filename

            with zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED) as zipf:
                for file_path in self.file_paths:
                    zipf.write(file_path)
        return zip_path

    def upload(self, zip_path: Path, deploy_config: DeployConfig) -> None:
        with log_task(
            start_message="Deploying bundle...", end_message="Bundle deployed"
        ):
            with open(zip_path, "rb") as zip_file:
                files = {
                    "file": zip_file,
                    "json_data": (None, deploy_config.model_dump_json()),
                }
                resp = requests.post(
                    self.deploy_url,
                    headers=self.headers,
                    files=files,
                )
            if resp.status_code != 200:
                log_error("Failed to trigger the deploy")
                sys.exit(1)

    def register_services(self) -> list[ServiceConfig]:
        services: list[ServiceConfig] = []
        service_names: set[str] = set()
        for file_path in self.file_paths:
            module_name = str(file_path).replace(os.path.sep, ".").replace(".py", "")
            spec = importlib.util.spec_from_file_location(file_path.stem, file_path)
            if spec and spec.loader:
                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)
                for name, obj in inspect.getmembers(module):
                    if inspect.isfunction(obj) and hasattr(obj, "_pare_register"):
                        try:
                            service_registration = cast(
                                ServiceRegistration,
                                obj._pare_register(),  # type: ignore
                            )
                            if name in service_names:
                                raise ValueError(f"Duplicate endpoint {name}")
                            service_names.add(name)
                            services.append(
                                ServiceConfig(
                                    name=service_registration.name,
                                    path=f"{module_name}:{service_registration.function}",
                                    requirements=service_registration.dependencies,
                                )
                            )
                        except Exception as e:
                            print(f"Error calling _pare_register on {name}: {e}")
                            break
            else:
                print(f"Couldn't load module from {file_path}")
        return services

    def deploy(self):
        console = Console()
        console.print(
            "[bold white]Deploying...[/bold white]",
        )
        self.validate_file_paths()
        services = self.register_services()
        deploy_config = DeployConfig(
            git_hash=get_current_git_hash(),
            python_version=PYTHON_VERSION,
            services=services,
            environment_variables=self.environment_variables,
        )
        with tempfile.TemporaryDirectory() as temp_dir:
            zip_path = self.bundle(temp_dir)
            self.upload(zip_path, deploy_config=deploy_config)
        console.print("[bold green]Deployed![/bold green]")



from __future__ import annotations

from contextlib import contextmanager
from datetime import datetime
from typing import TYPE_CHECKING

from rich.console import Console

if TYPE_CHECKING:
    from collections.abc import Generator


@contextmanager
def log_task(start_message: str, end_message: str = "") -> Generator[None, None, None]:
    console = Console()
    with console.status(
        f"      {start_message}", spinner="aesthetic", spinner_style="blue"
    ):
        try:
            # Before entering the block
            yield
        finally:
            if end_message:
                # After exiting the block
                # Format the current time to include leading zeros (HH:MM:SS)
                timestamp_str = datetime.now().strftime("[%H:%M:%S]")
                console.print(
                    f"{timestamp_str} [bright_green]✓[/bright_green] {end_message}",
                    highlight=False,
                )


def log_error(message: str) -> None:
    console = Console()
    console.print(f"[bright_red]✗ Error[/bright_red]: {message}")


def log_warning(message: str) -> None:
    console = Console()
    console.print(f"[yellow] Warning[/yellow]: {message}")



from __future__ import annotations

from typing import TYPE_CHECKING

import pytest

from pare.cli.main import create_parser

if TYPE_CHECKING:
    from pytest import CaptureFixture


def test_parser_deploy():
    parser = create_parser()
    args = parser.parse_args(["deploy", "test_file.py"])
    assert args.command == "deploy"
    assert args.file_patterns == ["test_file.py"]
    args = parser.parse_args(["deploy", "test_file.py", "test_other.py"])
    assert args.file_patterns == ["test_file.py", "test_other.py"]


def test_parser_deploy_no_file(capsys: CaptureFixture[str]):
    parser = create_parser()
    with pytest.raises(SystemExit) as exc:
        parser.parse_args(["deploy"])
    assert exc.value.code == 2

    captured = capsys.readouterr()
    assert "the following arguments are required: file" in captured.err


def test_parser_status():
    parser = create_parser()
    args = parser.parse_args(["status"])
    assert args.command == "status"



from __future__ import annotations

from pathlib import Path
from unittest.mock import MagicMock, patch

import pytest

from pare.cli.deploy import DeployHandler
from pare.models import ServiceConfig


@pytest.fixture
def temp_directory(tmp_path) -> Path:
    # Create temporary files
    (tmp_path / "file1.py").touch()
    (tmp_path / "file2.py").touch()
    return tmp_path


@pytest.fixture
def deploy_handler(temp_directory):
    return DeployHandler([str(temp_directory / "*.py")])


def test_init(temp_directory):
    handler = DeployHandler([str(temp_directory / "*.py")])
    assert len(handler.file_paths) == 2
    assert all(file.name in ["file1.py", "file2.py"] for file in handler.file_paths)


def test_bundle(deploy_handler, tmp_path):
    with patch("zipfile.ZipFile") as mock_zipfile:
        result = deploy_handler.bundle(tmp_path)
        assert isinstance(result, Path)
        assert result.name.endswith(".zip")
        mock_zipfile.assert_called_once()


@patch.object(DeployHandler, "register_services")
@patch.object(DeployHandler, "bundle")
@patch.object(DeployHandler, "upload")
def test_deploy(
    mock_upload: MagicMock,
    mock_bundle: MagicMock,
    mock_register: MagicMock,
    deploy_handler: DeployHandler,
) -> None:
    mock_register.return_value = [
        ServiceConfig(name="test", path="test", requirements=[])
    ]
    mock_bundle.return_value = Path("test.zip")

    deploy_handler.deploy()

    mock_register.assert_called_once()
    mock_bundle.assert_called_once()
    mock_upload.assert_called_once()



from __future__ import annotations

import os

from pare.sdk.main import endpoint


@endpoint(name="name1")
def hello_world():
    if "PARE_TEST_ONE" in os.environ:
        return f"[{os.environ["PARE_TEST_ONE"]}] Hello World!"
    print("Hello World!")


@endpoint(name="name2", dependencies=["pydantic==2.8.2"])
def test(echo: str):
    if "PARE_TEST_TWO" in os.environ:
        return f"[{os.environ["PARE_TEST_TWO"]}] {echo}"
    return echo



from __future__ import annotations

import json
from dataclasses import asdict, dataclass, field
from typing import Any, Callable, TypeVar

import aiohttp
import requests
from typing_extensions import ParamSpec

from pare import errors, settings
from pare.client import get_client_headers
from pare.models import ServiceRegistration


@dataclass
class RemoteInvocationArguments:
    args: list[Any] = field(default_factory=list)
    kwargs: dict[Any, Any] = field(default_factory=dict)


def invoke_endpoint(function_name: str, arguments: RemoteInvocationArguments) -> Any:
    try:
        response = requests.post(
            f"{settings.PARE_API_URL}/{settings.PARE_API_VERSION}{settings.PARE_API_INVOKE_URL_PATH}{function_name}/",
            headers=get_client_headers(),
            json=json.dumps(asdict(arguments)),
        )
        response.raise_for_status()
        json_response = response.json()
        return json_response.get("result")
    except requests.HTTPError as e:
        raise errors.PareInvokeError(
            f"Function invocation for '{function_name}' failed with status: {e.response.status_code}"
        )
    except Exception as e:
        raise errors.PareInvokeError(
            f"Could not invoke function: '{function_name}' due to error:\n{e}"
        )


async def async_invoke_endpoint(
    function_name: str, arguments: RemoteInvocationArguments
) -> Any:
    async with aiohttp.ClientSession() as session:
        try:
            async with session.post(
                f"{settings.PARE_API_URL}/{settings.PARE_API_VERSION}/invoke/{function_name}/",
                headers=get_client_headers(),
                json=json.dumps(asdict(arguments)),
            ) as response:
                response.raise_for_status()
                json_response = await response.json()
                return json_response.get("result")
        except aiohttp.ClientResponseError as e:
            raise errors.PareInvokeError(
                f"Function invocation for '{function_name}' failed with status: {e.status}"
            )
        except Exception as e:
            raise errors.PareInvokeError(
                f"Could not invoke function: '{function_name}' due to error:\n{e}"
            )


P = ParamSpec("P")
R = TypeVar("R")


def endpoint(name: str, dependencies: list[str] = []) -> Callable[..., Any]:
    def endpoint_decorator(
        function: Callable[P, R],
    ) -> Callable[P, R]:
        def _pare_register() -> ServiceRegistration:
            return ServiceRegistration(
                name=name, function=function.__name__, dependencies=dependencies
            )

        function._pare_register = _pare_register  # pyright: ignore[reportFunctionMemberAccess]

        def _as_lambda_handler() -> Callable[[Any, Any], Any]:
            def _lambda_handler(event: Any, context: Any) -> Any:
                if not isinstance(event, dict):
                    return {
                        "status": 400,
                        "detail": "Could not parse incoming data. The request body must be JSON.",
                    }
                if "args" not in event and "kwargs" not in event:
                    return {
                        "status": 400,
                        "detail": "Incoming JSON should contain 'args' or 'kwargs' to invoke the function.",
                    }
                try:
                    return {
                        "status": 200,
                        "result": function(
                            *event.get("args", []),  # type: ignore
                            **event.get("kwargs", {}),  # type: ignore
                        ),
                    }
                except Exception as e:
                    return {"status": 500, "detail": str(e)}

            return _lambda_handler

        function.as_lambda_function_url_handler = _as_lambda_handler  # pyright: ignore[reportFunctionMemberAccess]

        def _invoke_fn(*args: P.args, **kwargs: P.kwargs) -> Callable[P, R]:
            return invoke_endpoint(
                name,
                RemoteInvocationArguments(args=args, kwargs=kwargs),  # type: ignore
            )

        function.invoke = _invoke_fn  # pyright: ignore[reportFunctionMemberAccess]

        async def _async_invoke_fn(*args: P.args, **kwargs: P.kwargs) -> Callable[P, R]:
            return await async_invoke_endpoint(
                name,
                RemoteInvocationArguments(args=args, kwargs=kwargs),  # type: ignore
            )

        function.invoke_async = _async_invoke_fn  # pyright: ignore[reportFunctionMemberAccess]

        return function

    return endpoint_decorator



from __future__ import annotations

from pydantic import BaseModel, Field


class ServiceRegistration(BaseModel):
    name: str
    function: str
    dependencies: list[str] = Field(default_factory=list)


class ServiceConfig(BaseModel):
    name: str
    path: str
    requirements: list[str] = Field(default_factory=list)


class DeployConfig(BaseModel):
    git_hash: str
    python_version: str
    environment_variables: dict[str, str] = Field(default_factory=dict)
    services: list[ServiceConfig] = Field(default_factory=list)



from __future__ import annotations

from datetime import datetime
from typing import Any

import requests
from rich import box
from rich.console import Console
from rich.panel import Panel
from rich.table import Table

from pare import settings


def show_status():
    list_url = f"{settings.PARE_API_URL}/{settings.PARE_API_VERSION}{settings.PARE_API_SERVICES_URL_PATH}"
    headers = {settings.PARE_API_KEY_HEADER: settings.PARE_API_KEY}
    try:
        response = requests.get(list_url, headers=headers)
        response.raise_for_status()
        display_status_table(response.json())
    except Exception as e:
        console = Console()
        console.print(f"[bold red]Error: {e}[/bold red]")


def display_status_table(status_data: list[dict[str, Any]]):
    console = Console()

    if not status_data:
        empty_message = Panel(
            "[italic]No deployment data available.[/italic]\n\n[dim]Try deploying a service or check your connection.[/dim]",
            title="Deployment Data",
            expand=False,
            border_style="white",
            box=box.ROUNDED,
        )
        console.print(empty_message)
        return

    table = Table(title="Deployment Data")
    table.add_column("Name", style="cyan", no_wrap=True)
    table.add_column("Git Hash", style="magenta")
    table.add_column("Created At", style="yellow")

    for item in status_data:
        name = item["name"]
        git_hash = item["deployment"]["git_hash"]
        created_at = datetime.fromisoformat(item["created_at"]).strftime(
            "%Y-%m-%d %H:%M:%S"
        )

        table.add_row(name, git_hash, created_at)

    console.print(table)



from __future__ import annotations

import subprocess
import sys
from functools import lru_cache

from pare import settings
from pare.console import log_error, log_warning

# TODO: build a Client singleton to handle all requests


@lru_cache(maxsize=1)
def get_current_git_hash() -> str:
    if settings.PARE_GIT_HASH:
        if len(settings.PARE_GIT_HASH) > 7:
            log_warning(
                "PARE_GIT_HASH is longer than 7 characters. Using the first 7 characters."
            )
        return settings.PARE_GIT_HASH[:7]

    # if the git hash is not provided, we will try to get it from the current git state
    try:
        # verify we are in a clean git state
        subprocess.check_call(["git", "diff", "--quiet"])
    except subprocess.CalledProcessError:
        log_warning("Git state is not clean. Using the last commit hash.")

    try:
        return (
            subprocess.check_output(["git", "rev-parse", "HEAD"]).decode().strip()[:7]
        )
    except subprocess.CalledProcessError:
        log_error("Pare failed to get the current git hash.")
        sys.exit(1)


def get_client_headers() -> dict[str, str]:
    headers = {settings.PARE_API_KEY_HEADER: settings.PARE_API_KEY}
    if settings.PARE_ATOMIC_DEPLOYMENT_ENABLED:
        headers[settings.PARE_ATOMIC_DEPLOYMENT_HEADER] = get_current_git_hash()
    return headers



from __future__ import annotations

import sys
from pathlib import Path
from time import sleep
from typing import Any

import requests

from pare import settings
from pare.console import log_task
from pare.constants import GITHUB_CLIENT_ID


def parse_response(response: requests.Response) -> dict[str, Any]:
    if response.status_code in [200, 201]:
        return response.json()
    elif response.status_code == 401:
        print("Authorization failed.")
        sys.exit(1)
    else:
        print(response)
        print(response.text)
        sys.exit(1)


def request_device_code() -> dict[str, Any]:
    url = "https://github.com/login/device/code"
    params = {"client_id": GITHUB_CLIENT_ID}
    headers = {"Accept": "application/json"}
    response = requests.post(url, data=params, headers=headers)
    return parse_response(response)


def request_token(device_code: str) -> dict[str, Any]:
    url = "https://github.com/login/oauth/access_token"
    params = {
        "client_id": GITHUB_CLIENT_ID,
        "device_code": device_code,
        "grant_type": "urn:ietf:params:oauth:grant-type:device_code",
    }
    headers = {"Accept": "application/json"}
    response = requests.post(url, data=params, headers=headers)
    return parse_response(response)


def poll_for_token(device_code: str, interval: int) -> str:
    error_message: str = ""
    with log_task(start_message="Polling for token..."):
        while True:
            response = request_token(device_code)
            error = response.get("error", "")
            access_token = response.get("access_token", "")

            if error:
                if error == "authorization_pending":
                    sleep(interval)
                    continue
                elif error == "slow_down":
                    sleep(interval + 2)
                    continue
                elif error == "expired_token":
                    error_message = (
                        "The device code has expired. Please run `login` again."
                    )
                    break
                elif error == "access_denied":
                    error_message = "Login cancelled by user."
                    break
                else:
                    error_message = "An unknown error occurred."
                    break
            elif access_token:
                break

    if error_message:
        print(error_message)
        sys.exit(1)

    return access_token


def login_to_pare(token: str) -> str:
    url = f"{settings.PARE_API_URL}/login-with-github/"
    headers = {"Content-Type": "application/json"}
    data = {"token": token}
    response = requests.post(url, json=data, headers=headers)
    response = parse_response(response)
    try:
        api_key = response["api_key"]
    except KeyError:
        print("Login failed.")
        sys.exit(1)
    print("Successfully logged in!")
    return api_key


def stash_api_key(api_key: str) -> None:
    api_key_path = Path(settings.PARE_API_KEY_FILE)
    api_key_path.parent.mkdir(parents=True, exist_ok=True)
    api_key_path.write_text(api_key)


def check_already_has_api_key() -> bool:
    api_key_path = Path(settings.PARE_API_KEY_FILE)
    return api_key_path.exists() and bool(api_key_path.read_text())


def login():
    if check_already_has_api_key():
        confirm = input(
            f"You already have an API key (at '{settings.PARE_API_KEY_FILE}'). Do you want to overwrite it? (y/n): "
        )
        if confirm.lower() != "y":
            print("Exiting...")
            sys.exit(0)

    device_code_response = request_device_code()
    verification_uri = device_code_response["verification_uri"]
    user_code = device_code_response["user_code"]
    device_code = device_code_response["device_code"]
    interval = device_code_response["interval"]

    print(f"Please visit: {verification_uri}")
    print(f"and enter code: {user_code}")

    token = poll_for_token(device_code, interval)
    api_key = login_to_pare(token)
    stash_api_key(api_key)
    print("Successfully authenticated!")



from __future__ import annotations

import sys

import requests

from pare import settings
from pare.console import log_error


def delete_function(function_name: str, git_hash: str) -> None:
    headers = {settings.PARE_API_KEY_HEADER: settings.PARE_API_KEY}
    if git_hash:
        headers[settings.PARE_ATOMIC_DEPLOYMENT_HEADER] = git_hash

    try:
        response = requests.delete(
            f"{settings.PARE_API_URL}/{settings.PARE_API_VERSION}{settings.PARE_API_DELETE_URL_PATH}{function_name}/",
            headers=headers,
        )
        response.raise_for_status()
    except requests.HTTPError as e:
        log_error(f"Failed to delete function. Status code: {e.response.status_code}")
        sys.exit(1)
    except Exception as e:
        log_error(f"Failed to delete function due to error: {e}")
        sys.exit(1)



from __future__ import annotations

from pare import endpoint
from pare.models import ServiceRegistration


def test_default_endpoint_decorator():
    @endpoint(name="name")
    def test_function():
        return "testy"

    assert test_function() == "testy"
    assert hasattr(test_function, "_pare_register")
    assert test_function._pare_register() == ServiceRegistration(
        name="name",
        dependencies=[],
        function="test_function",
    )


def test_custom_endpoint_decorator():
    @endpoint(name="name2", dependencies=["fastapi"])
    def test_function():
        return "test2"

    assert test_function() == "test2"
    assert hasattr(test_function, "_pare_register")
    assert test_function._pare_register() == ServiceRegistration(
        name="name2",
        dependencies=["fastapi"],
        function="test_function",
    )


def test_as_lambda_handler():
    @endpoint(name="name3", dependencies=["fastapi"])
    def test_function():
        return "test3"

    assert test_function() == "test3"
    assert hasattr(test_function, "as_lambda_function_url_handler")
    handler_result = test_function.as_lambda_function_url_handler()({}, {})
    assert handler_result.get("status") == 400

    handler_result = test_function.as_lambda_function_url_handler()({"args": []}, {})
    assert handler_result.get("status") == 200
    assert handler_result.get("result") == "test3"



from __future__ import annotations

import argparse
import sys

from rich.console import Console

from pare import settings
from pare.cli.delete import delete_function
from pare.cli.deploy import DeployHandler
from pare.cli.status import show_status
from pare.login import login


def verify_logged_in() -> None:
    if not settings.PARE_API_KEY:
        console = Console()
        console.print(
            "[bold red]Error:[/bold red] Pare API key not found. Try 'pare login' first."
        )
        sys.exit(1)


def parse_env_vars(env_vars: list[str]) -> dict[str, str]:
    environment_variables: dict[str, str] = {}
    try:
        for env_var in env_vars:
            key, value = env_var.split("=")
            environment_variables[key] = value
    except ValueError:
        raise ValueError("Environment variables must be in the format KEY=VALUE")
    return environment_variables


def deploy(file_patterns: list[str], env_vars: list[str]) -> None:
    verify_logged_in()
    environment_variables = parse_env_vars(env_vars)
    DeployHandler(
        file_patterns=file_patterns, environment_variables=environment_variables
    ).deploy()


def delete(function_name: str, git_hash: str = "", force: bool = False) -> None:
    verify_logged_in()
    console = Console()
    if not force and (
        console.input(
            f"You are about to delete your deployed function called [bold red]'{function_name}'[/bold red]. "
            "Type the function name to confirm: "
        )
        != function_name
    ):
        console.print("[bold white]Operation cancelled.[/bold white]")
        return

    delete_function(function_name, git_hash=git_hash)
    if git_hash:
        console.print(
            f"[bold red]'{function_name}' (git hash: '{git_hash}') deleted.[/bold red]"
        )
    else:
        console.print(f"[bold red]'{function_name}' deleted.[/bold red]")


def status() -> None:
    verify_logged_in()
    show_status()


def create_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        description="A CLI tool to deploy python lambdas with a single command"
    )

    subparsers = parser.add_subparsers(dest="command")
    subparsers.required = True

    subparsers.add_parser("login", help="Authenticate with github.")

    parser_deploy = subparsers.add_parser("deploy", help="Deploy the application.")
    parser_deploy.add_argument(
        "file_patterns",
        nargs="+",
        type=str,
        help="One or more file patterns to include in the deployed bundle.",
    )
    parser_deploy.add_argument(
        "-e",
        "--env",
        action="append",
        dest="env_vars",
        help="Specify an environment variable in the format KEY=VALUE. Can be used multiple times.",
    )

    subparsers.add_parser("status", help="Check the status of your deployed functions.")

    parser_delete = subparsers.add_parser("delete", help="Delete a deployed function.")
    parser_delete.add_argument(
        "function_name", type=str, help="The name of the function to delete."
    )
    parser_delete.add_argument(
        "-g",
        "--git-hash",
        nargs="?",
        type=str,
        help="The git hash of the function to delete.",
    )
    parser_delete.add_argument(
        "--force",
        action="store_true",
        help="Skip the confirmation prompt and delete the function.",
    )

    return parser


def main() -> None:
    parser = create_parser()
    args = parser.parse_args()
    if args.command == "deploy":
        deploy(args.file_patterns, env_vars=args.env_vars or [])
    elif args.command == "status":
        status()
    elif args.command == "delete":
        delete(args.function_name, git_hash=args.git_hash, force=args.force)
    elif args.command == "login":
        login()
    else:
        print("Unknown command")
        parser.print_help()


if __name__ == "__main__":
    main()
